"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _reactNativeInappbrowserReborn = _interopRequireDefault(require("react-native-inappbrowser-reborn"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class Adapter {
  constructor(kcClient, inAppBrowserOptions) {
    this.kcClient = kcClient;
    this.inAppBrowserOptions = inAppBrowserOptions;
  }
  /**
   * Start login process
   *
   * @param {KeycloakLoginOptions} options Login options
   */


  async login(options) {
    const loginUrl = this.kcClient.createLoginUrl(options);

    if (await _reactNativeInappbrowserReborn.default.isAvailable()) {
      // See for more details https://github.com/proyecto26/react-native-inappbrowser#authentication-flow-using-deep-linking
      const res = await _reactNativeInappbrowserReborn.default.openAuth(loginUrl, this.kcClient.redirectUri, this.inAppBrowserOptions);

      if (res.type === 'success' && res.url) {
        const oauth = this.kcClient.parseCallback(res.url);
        return this.kcClient.processCallback(oauth);
      }

      throw new Error('Authentication flow failed');
    } else {
      throw new Error('InAppBrowser not available'); // TODO: maybe!
      //   Linking.openURL(loginURL);
    }
  }

  async logout(options) {
    const logoutUrl = this.kcClient.createLogoutUrl(options);

    if (await _reactNativeInappbrowserReborn.default.isAvailable()) {
      // See for more details https://github.com/proyecto26/react-native-inappbrowser#authentication-flow-using-deep-linking
      const res = await _reactNativeInappbrowserReborn.default.openAuth(logoutUrl, this.kcClient.redirectUri, this.inAppBrowserOptions);

      if (res.type === 'success') {
        return this.kcClient.clearToken();
      }

      throw new Error('Logout flow failed');
    } else {
      throw new Error('InAppBrowser not available'); // TODO: maybe!
      //   Linking.openURL(logoutUrl);
    }
  }

  async register(options) {
    const registerUrl = this.kcClient.createRegisterUrl(options);

    if (await _reactNativeInappbrowserReborn.default.isAvailable()) {
      // See for more details https://github.com/proyecto26/react-native-inappbrowser#authentication-flow-using-deep-linking
      const res = await _reactNativeInappbrowserReborn.default.openAuth(registerUrl, this.kcClient.redirectUri, this.inAppBrowserOptions);

      if (res.type === 'success' && res.url) {
        const oauth = this.kcClient.parseCallback(res.url);
        return this.kcClient.processCallback(oauth);
      }

      throw new Error('Registration flow failed');
    } else {
      throw new Error('InAppBrowser not available'); // TODO: maybe!
      //   Linking.openURL(registerUrl);
    }
  }

  async accountManagement() {
    const accountUrl = this.kcClient.createAccountUrl();

    if (typeof accountUrl !== 'undefined') {
      await _reactNativeInappbrowserReborn.default.open(accountUrl, this.inAppBrowserOptions);
    } else {
      throw 'Not supported by the OIDC server';
    }
  }

  redirectUri(options) {
    if (options && options.redirectUri) {
      return options.redirectUri;
    }

    if (this.kcClient.redirectUri) {
      return this.kcClient.redirectUri;
    }

    return ''; // TODO: Retrieve app deeplink
  }

}

var _default = Adapter;
exports.default = _default;
//# sourceMappingURL=adapter.js.map